# -*- coding: utf-8 -*-
"""
    sphinxcontrib.websupport.storage.differ
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    A differ for creating an HTML representations of proposal diffs

    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

import re
from difflib import Differ

from sphinx.util.pycompat import htmlescape


class CombinedHtmlDiff(object):
    """Create an HTML representation of the differences between two pieces
    of text.
    """
    highlight_regex = re.compile(r'([\+\-\^]+)')

    def __init__(self, source, proposal):
        proposal = htmlescape(proposal)

        differ = Differ()
        self.diff = list(differ.compare(source.splitlines(1),
                                        proposal.splitlines(1)))

    def make_text(self):
        return '\n'.join(self.diff)

    def make_html(self):
        """Return the HTML representation of the differences between
        `source` and `proposal`.

        :param source: the original text
        :param proposal: the proposed text
        """
        html = []
        diff = self.diff[:]
        line = diff.pop(0)
        next = diff.pop(0)
        while True:
            html.append(self._handle_line(line, next))
            line = next
            try:
                next = diff.pop(0)
            except IndexError:
                html.append(self._handle_line(line))
                break
        return ''.join(html).rstrip()

    def _handle_line(self, line, next=None):
        """Handle an individual line in a diff."""
        prefix = line[0]
        text = line[2:]

        if prefix == ' ':
            return text
        elif prefix == '?':
            return ''

        if next is not None and next[0] == '?':
            tag = prefix == '+' and 'ins' or 'del'
            text = self._highlight_text(text, next, tag)
        css_class = prefix == '+' and 'prop-added' or 'prop-removed'

        return '<span class="%s">%s</span>\n' % (css_class, text.rstrip())

    def _highlight_text(self, text, next, tag):
        """Highlight the specific changes made to a line by adding
        <ins> and <del> tags.
        """
        next = next[2:]
        new_text = []
        start = 0
        for match in self.highlight_regex.finditer(next):
            new_text.append(text[start:match.start()])
            new_text.append('<%s>' % tag)
            new_text.append(text[match.start():match.end()])
            new_text.append('</%s>' % tag)
            start = match.end()
        new_text.append(text[start:])
        return ''.join(new_text)



# -*- coding: utf-8 -*-
"""
    sphinxcontrib.websupport.search.whooshsearch
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Whoosh search adapter.

    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

from whoosh import index
from whoosh.fields import Schema, ID, TEXT
from whoosh.qparser import QueryParser
from whoosh.analysis import StemmingAnalyzer

from six import text_type

from sphinx.util.osutil import ensuredir
from sphinxcontrib.websupport.search import BaseSearch


class WhooshSearch(BaseSearch):
    """The whoosh search adapter for sphinx web support."""

    # Define the Whoosh Schema for the search index.
    schema = Schema(path=ID(stored=True, unique=True),
                    title=TEXT(field_boost=2.0, stored=True),
                    text=TEXT(analyzer=StemmingAnalyzer(), stored=True))

    def __init__(self, db_path):
        ensuredir(db_path)
        if index.exists_in(db_path):
            self.index = index.open_dir(db_path)
        else:
            self.index = index.create_in(db_path, schema=self.schema)
        self.qparser = QueryParser('text', self.schema)

    def init_indexing(self, changed=[]):
        for changed_path in changed:
            self.index.delete_by_term('path', changed_path)
        self.index_writer = self.index.writer()

    def finish_indexing(self):
        self.index_writer.commit()

    def add_document(self, pagename, filename, title, text):
        self.index_writer.add_document(path=text_type(pagename),
                                       title=title,
                                       text=text)

    def handle_query(self, q):
        searcher = self.index.searcher()
        whoosh_results = searcher.search(self.qparser.parse(q))
        results = []
        for result in whoosh_results:
            context = self.extract_context(result['text'])
            results.append((result['path'],
                            result.get('title', ''),
                            context))
        return results



# -*- coding: utf-8 -*-
"""
    sphinxcontrib.websupport.errors
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Contains Error classes for the web support package.

    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""


class DocumentNotFoundError(Exception):
    pass


class UserNotAuthorizedError(Exception):
    pass


class CommentNotAllowedError(Exception):
    pass


class NullSearchException(Exception):
    pass



# -*- coding: utf-8 -*-
"""
    sphinxcontrib.websupport.storage
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Storage for the websupport package.

    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""


class StorageBackend(object):
    def pre_build(self):
        """Called immediately before the build process begins. Use this
        to prepare the StorageBackend for the addition of nodes.
        """
        pass

    def has_node(self, id):
        """Check to see if a node exists.

        :param id: the id to check for.
        """
        raise NotImplementedError()

    def add_node(self, id, document, source):
        """Add a node to the StorageBackend.

        :param id: a unique id for the comment.
        :param document: the name of the document the node belongs to.
        :param source: the source files name.
        """
        raise NotImplementedError()

    def post_build(self):
        """Called after a build has completed. Use this to finalize the
        addition of nodes if needed.
        """
        pass

    def add_comment(self, text, displayed, username, time,
                    proposal, node_id, parent_id, moderator):
        """Called when a comment is being added.

        :param text: the text of the comment
        :param displayed: whether the comment should be displayed
        :param username: the name of the user adding the comment
        :param time: a date object with the time the comment was added
        :param proposal: the text of the proposal the user made
        :param node_id: the id of the node that the comment is being added to
        :param parent_id: the id of the comment's parent comment.
        :param moderator: whether the user adding the comment is a moderator
        """
        raise NotImplementedError()

    def delete_comment(self, comment_id, username, moderator):
        """Delete a comment.

        Raises :class:`~sphinxcontrib.websupport.errors.UserNotAuthorizedError`
        if moderator is False and `username` doesn't match the username
        on the comment.

        :param comment_id: The id of the comment being deleted.
        :param username: The username of the user requesting the deletion.
        :param moderator: Whether the user is a moderator.
        """
        raise NotImplementedError()

    def get_metadata(self, docname, moderator):
        """Get metadata for a document. This is currently just a dict
        of node_id's with associated comment counts.

        :param docname: the name of the document to get metadata for.
        :param moderator: whether the requester is a moderator.
        """
        raise NotImplementedError()

    def get_data(self, node_id, username, moderator):
        """Called to retrieve all data for a node. This should return a
        dict with two keys, *source* and *comments* as described by
        :class:`~sphinxcontrib.websupport.WebSupport`'s
        :meth:`~sphinxcontrib.websupport.WebSupport.get_data` method.

        :param node_id: The id of the node to get data for.
        :param username: The name of the user requesting the data.
        :param moderator: Whether the requestor is a moderator.
        """
        raise NotImplementedError()

    def process_vote(self, comment_id, username, value):
        """Process a vote that is being cast. `value` will be either -1, 0,
        or 1.

        :param comment_id: The id of the comment being voted on.
        :param username: The username of the user casting the vote.
        :param value: The value of the vote being cast.
        """
        raise NotImplementedError()

    def update_username(self, old_username, new_username):
        """If a user is allowed to change their username this method should
        be called so that there is not stagnate data in the storage system.

        :param old_username: The username being changed.
        :param new_username: What the username is being changed to.
        """
        raise NotImplementedError()

    def accept_comment(self, comment_id):
        """Called when a moderator accepts a comment. After the method is
        called the comment should be displayed to all users.

        :param comment_id: The id of the comment being accepted.
        """
        raise NotImplementedError()
